/****************************************************************************
**
** Copyright (C) 1992-2009 Nokia. All rights reserved.
**
** This file is part of Qt Jambi.
**
** $BEGIN_LICENSE$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** $END_LICENSE$

**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef GAMEGRAMMAR_H
#define GAMEGRAMMAR_H

#include <QtCore/QString>
#include <QtCore/QHash>
#include <QtCore/QStringList>

class GameScene ;
class GameAction ;
class AbstractGameObject ;

class GameGrammar
{
public:
    GameGrammar(GameScene *scene);
    virtual ~GameGrammar();

    void registerGameObject(AbstractGameObject *gameObject);
    void addNameToGameObject(AbstractGameObject *gameObject, const QString &other_name);
    void addVerb(const QString &verb, GameAction *action)
    {
        m_actions[verb] = action;
    }

    virtual void parse(const QString &command);

protected:
    virtual GameAction *command();
    virtual GameAction *verb();
    virtual AbstractGameObject *object();

    virtual bool filler();
    virtual bool and_token();
    virtual QString currentToken(int token_count = 1) const;
    virtual void nextToken();

    QHash<QString, AbstractGameObject *> m_objects;
    QHash<QString, GameAction *> m_actions;
    GameScene *m_scene;
    QStringList m_current_command;
};

#endif
