/****************************************************************************
**
** Copyright (C) 1992-2009 Nokia. All rights reserved.
** Copyright 2005 Harald Fernengel <harry@kdevelop.org>
**
** This file is part of Qt Jambi.
**
** $BEGIN_LICENSE$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** $END_LICENSE$

**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef PREPROCESSOR_H
#define PREPROCESSOR_H

#include <QtCore/qglobal.h>
#include <QtCore/qstring.h>
#include <QtCore/qstringlist.h>

#include "rpp/pp-environment.h"
#include "pp-engine-bits.h"
#include "util.h"

class PreprocessorPrivate {
    public:
        QByteArray result;
        rpp::pp_environment env;
        QStringList includePaths;

        void initPP(rpp::pp &proc) {
            foreach(QString path, includePaths)
                proc.push_include_path(::toStdString(path));
        }
};

class Preprocessor {
    public:
        Preprocessor();
        ~Preprocessor();

        void processFile(const QString &fileName);
        void processString(const QByteArray &str);

        void addIncludePaths(const QStringList &includePaths);

        QByteArray result() const;

        QStringList macroNames() const;

        struct MacroItem {
            QString name;
            QStringList parameters;
            QString definition;
            bool isFunctionLike;
#ifdef PP_WITH_MACRO_POSITION
            QString fileName;
#endif
        };
        QList<MacroItem> macros() const;

    private:
        Q_DISABLE_COPY(Preprocessor)
        PreprocessorPrivate *d;
};

#endif
