/****************************************************************************
**
** Copyright (C) 1992-2009 Nokia. All rights reserved.
**
** This file is part of Qt Jambi.
**
** $BEGIN_LICENSE$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** $END_LICENSE$

**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "reporthandler.h"
#include "typesystem/typedatabase.h"

int ReportHandler::m_warning_count = 0;
int ReportHandler::m_suppressed_count = 0;
QString ReportHandler::m_context;
ReportHandler::DebugLevel ReportHandler::m_debug_level = NoDebug;
QSet<QString> ReportHandler::m_reported_warnings;


void ReportHandler::warning(const QString &text) {
    QString warningText = QString("WARNING(%1) :: %2").arg(m_context).arg(text);

    TypeDatabase *db = TypeDatabase::instance();
    if (db && db->isSuppressedWarning(warningText)) {
        ++m_suppressed_count;
    } else if (!m_reported_warnings.contains(warningText)) {
        qDebug("%s", qPrintable(warningText));
        ++m_warning_count;

        m_reported_warnings.insert(warningText);
    }
}

void ReportHandler::debug(DebugLevel level, const QString &text) {
    if (m_debug_level == NoDebug)
        return;

    if (level <= m_debug_level)
        qDebug(" - DEBUG(%s) :: %s", qPrintable(m_context), qPrintable(text));
}
