/****************************************************************************
**
** Copyright (C) 1992-2009 Nokia. All rights reserved.
**
** This file is part of Qt Jambi.
**
** $BEGIN_LICENSE$
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
** $END_LICENSE$

**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

package com.trolltech.qt.core;

import com.trolltech.qt.gui.QApplication;

/**
 * The QSynchronousInvokable class is used internally by QCoreApplication.invokeLaterAndWait()
 * to synchronously execute a java.lang.Runnable from the gui-thread.
 *
 * @author eskil
 * @see QCoreApplication
 */
class QSynchronousInvokable extends QObject {
    static QEvent.Type SYNCHRONOUS_INVOKABLE_EVENT = QEvent.Type.resolve(QEvent.Type.User.value() + 1);

    private Runnable runnable;
    public QSynchronousInvokable(Runnable runnable) {
        disableGarbageCollection();
        if(QCoreApplication.instance().nativeId() != 0) {
            moveToThread(QCoreApplication.instance().thread());
            this.runnable = runnable;
        }

        if (runnable == null || Thread.currentThread().equals(QApplication.instance().thread()))
            invoked = true;

    }

    private Boolean invoked = false;
    synchronized void waitForInvoked() {
        while (!invoked) {
            try {
                wait();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        invoked = false;
    }

    @Override
    public final boolean event(com.trolltech.qt.core.QEvent e) {
        if (e.type() == SYNCHRONOUS_INVOKABLE_EVENT && runnable != null) {
            runnable.run();
            synchronized (this) {
                invoked = true;
                notifyAll();
            }
            return true;
        }
        return super.event(e);
    }
}
